/* Programme qui sert d'outil pour rechercher la solution technique
   la mieux adaptée pour commuter les enroulements des "magnétrons".
   La méthode consiste à générer un signal "rectangulaire" précis dont on peut à convenance
   ajuster la durée à l'état "1" et celle au niveau "0".
   L'afficheur OLED indique les deux durées ainsi que la fréquence qui en résulte.
   • Un codeur incrémental ajuste les durées. C'est son bouton central qui fait sortir du
     mode programmation.
   • Un inverseur branché sur D11 permet de choisir quel est le niveau traité lorsque
     l'on tourne le codeur incrémental.
   NOTE : Certains choix d'E/S peuvent sembler étranges mais ils anticipent sur une répartition
          qui envisage dès ce stade des études :
          • L'utilisation d'un BUZZER passif sur D11,
          • l'utilisation de trois LEDs ou d'une LRD triple sur A0, A1 et A2.
   
Branchements à effectuer :
//-------------------------- Entrées Analogiques -----------------------------
 A3 : Bouton central du codeur rotatif.
 A4 : SDA OLED.
 A5 : SCL OLED.
//----------------------- Entrées / Sorties BINAIRES ------------------------- 
 Codeur rotatif B : Broche D2. >>>>>>>>>>> ATTENTION : En fonction du codeur KY-40 approvisionné il
 Codeur rotatif A : Broche D3. >>>>>>>>>>>>>>>>>>>>>>> faudra éventuellement inverser D2 et D3 pour
                               >>>>>>>>>>>>>>>>>> obtenir un sens de variation correct.
 D4 : Inverseur pour choisir l'justement de "0" ou de "1".
 D13 : Sortie du signal pour étudier les circuit de commutation.
 
                                                   PGM               RAM dynamique
Compilateur Version 1.7.9                 13248 Octets (43%)     332 Octets (16%)    */

//--------------------- Déclarations pour le matériel -----------------------
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)
//------------- Définition des broches d'entrées analogiques ----------------
#define BP_central 17 // A3 pour lire le bouton central.

//------------- Définition des broches de sorties analogiques ---------------
#define LED_Clavier 13 // LED rouge d'Arduino.

//-------------- Définition des broches de sorties Binaires -----------------
#define Codeur_A 3    // D2 : Codeur rotatif.
#define Codeur_B 2    // D3 : Codeur rotatif.
#define Inverseur 11    // D11 : Choix du "0" ou du "1"..
#define Signal_Binaire 13 // Sortie du signal de pilotage.

//-------------------------- Variables du programme -------------------------
boolean A_set, B_set;   // État des sorties déphasées du codeur.
boolean BP_central_a_traiter;
boolean INT_active;     // Indique un INT_active d'état détecté.
boolean Sens_positif; // Sens de rotation.

int Duree_zero, Duree_un; // Durées en centièmes de seconde.

float Frequence, T0, T1;

unsigned long Temps_Ecoule_depuis_RESET = 0;

volatile boolean Rotation;  // Indique une activation du codeur rotatif.
//---------------------------------------------------------------------------

void setup() {
  //---------------------- Définition des vecteurs d'interruption ----------------------
  attachInterrupt(0, Front_sur_A, CHANGE); // Appel à Front_sur_A si INT_active d'état détecté.
  attachInterrupt(1, Front_sur_B, CHANGE); // Appel à Front_sur_B si INT_active d'état détecté.
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  pinMode(Codeur_A,INPUT); pinMode(Codeur_B,INPUT);
  pinMode(Inverseur,INPUT_PULLUP); // Forcé au +5Vcc par résistance interne.
  pinMode(Signal_Binaire, OUTPUT); digitalWrite(Signal_Binaire, LOW);
  //--------------------- Initialisation des variables ---------------------
  Duree_zero = 50; Duree_un = 10; // Dixièmes de secondes.
  BP_central_a_traiter = false; Rotation = false;
  PAGE1(); Front_sur_A(); Front_sur_B();} // Voir informations en fin de listage.

void loop() {
  if (Rotation)TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL();
  //================= Chronométrage pour l'état un ================
  digitalWrite(Signal_Binaire, HIGH); delay(Duree_un * 10);
  //=============== Chronométrage pour l'état zero ================
  digitalWrite(Signal_Binaire, LOW); delay(Duree_zero * 10);}

//-------------------- Routines de servitude pour traiter le capteur incrémental -------------------
  void Delay_antirebonds() {for(int T=0; T<1000; T++);} // Valeur de 100 expérimentale.
  
  void Front_sur_A(){ // Interruption quand A change d'état avant B.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds.
    if (digitalRead(Codeur_A) != A_set ) {// Test pour savoir si l'état a changé.
      A_set = !A_set;
      if (A_set && !B_set) {Sens_positif = true; Rotation = true; Delay_antirebonds();} } }
  
  void Front_sur_B(){ // Interruption quand B change d'état avant A.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds. 
    if( digitalRead(Codeur_B) != B_set ) {// Test pour savoir si l'état a changé.
      B_set = !B_set;
      // Décrémenter le compteur si B avant A.
      if (B_set && !A_set) {Sens_positif = false; Rotation = true; Delay_antirebonds();} } }
  //--------------------------------------------------------------------------------------------------
  
void PAGE1() {
  T0 = (float(Duree_zero ))/ 100; T1 = (float(Duree_un)) / 100;
  Frequence = (100 / float(Duree_zero + Duree_un));
  u8g.setFont(u8g_font_6x10); // Pocice de matrices de 6x10 pixels lettres MJA/min comprises.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //================ Définition des éléments à afficher ================
  CADRES();
  Aff_TEXTE(10, 9 ,"Signal Binaire D13.");
  Aff_TEXTE(25,26,"Etat 1 : "); u8g.print(T1,2); u8g.print(" s");
  Aff_TEXTE(25,41,"Etat 0 : "); u8g.print(T0,2); u8g.print(" s");
  Aff_TEXTE(6,55,"Frequence : "); u8g.print(Frequence,2); u8g.print(" Hz");}
  //====================================================================
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
 
void CADRES() {u8g.drawFrame(0,0,128,13); u8g.drawFrame(0,15,128,48);} // Cadre TITRE et cadre données.
  
void Aff_TEXTE(byte X, byte Y, char TEXTE[]) {
  char Caractere; byte I = 0;
  while (Caractere !=0) {Caractere = TEXTE[I]; u8g.setPrintPos((I*6)+X, Y); I++; u8g.print(Caractere);}}

void Teste_activation_BP_central() {
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
     // ========== Attendre le relacher du B.P. central. ==========
     while (analogRead(BP_central) < 512); // Attendre le relâché.
     // ===========================================================
     BP_central_a_traiter = true;} }

void TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL() {
  while (!BP_central_a_traiter) {
        if (Rotation) {
           if (digitalRead(Inverseur)) {
              if (Sens_positif) {if (Duree_un < 200) Duree_un = Duree_un + 5;}
              else {if (Duree_un > 5) Duree_un = Duree_un - 5;}}    
           else {if (Sens_positif) {if (Duree_zero < 500) Duree_zero = Duree_zero + 5;}
              else {if (Duree_zero > 5)Duree_zero = Duree_zero - 5;}}
        PAGE1(); Rotation = false; INT_active = true;}
        // Ci-dessus : Armer un nouveau comptage et Prise en compte du traitement.
        Teste_activation_BP_central();}
        BP_central_a_traiter = false;}

/* Problèmes liés au codeur rotatif :
   ----------------------------------
   Ce type de codeur engendre une rotation fantôme positive et négative à sa première utilisation.
   Du coup pour obtenir le signal sur un RESET il faut cliquer sur le bouton central. 
  
   En fonction du codeur rotatif approvisionné, il peut s'avérer nécessaire d'inverser D2 et D3
   pour obtenir le sens de déplacement correct.

   Ayant approvisionné ce type de codeur chez divers fournisseurs, les tarifs ne sont pas les mêmes,
   mais la qualité non plus. Les moins onéreux sont victimes de faux contacts qui se traduisent par
   des comptages aléatoires. Aussi, se c'est le cas pour le composant utilisé chez vous, un moyen
   radical d'éliminer ce problème particulièrement pénalisant consiste à augmenter "la constante de
   tems" de l'établissement du niveau électrique logique en plaçant un condendateur entre A, B et GND.
  Des valeurs de l'ordre de 10nF conviennent sans problème. */


