/* Programme qui sert d'outil pour rechercher la solution technique
   la mieux adaptée pour commuter en séquence les enroulements des "magnétrons".
   La méthode consiste à générer un signal "rectangulaire" précis dont on peut à convenance
   ajuster la durée à l'état "1" et celle au niveau "0". Puis, une séquence sans fin est
   générée sur les huit sorties D4 à D12.
   Noter que durant les états "0" du cycle les trois LEDs sont balayées rapidement.
   L'afficheur OLED indique les deux durées ainsi que la fréquence qui en résulte.
   
   >>>> ATTENTION : Alimenté en 16,5v le courant traversant les enroulements est de
   >>>>             2,75A soit une puissance dissipée de l'ordre de 2,75 x 16.5 / 8 = 5,7W.
   >>>>             De ce fait ils chauffent et il ne faut pas laisser le balayage en
   >>>>             permanence. Donc cliquer sur le BP central, observer pendant quelques
   >>>>             secondes le comportement puis rapidement tourner le codeur rotatif
   >>>>             pour replacer la machine en veille. (La bille également chauffe.)
   
   • Sur RESET la LED rouge du collisionneur est allumée et le système est en mode
     de saisie de la durée du "0" avec le codeur incrémental. C'est son bouton central qui fait
     sortir du mode programmation des durées. La LED rouge du collisionneur s'éteint.
   • Le codeur incrémental incrémente les durées dans le sens horaire et les diminue dans le
     sens antihoraire. (Il y a effet de butée dans le sens négatif.)
   • Si durant la rotation du codeur incrémental on clique sur le BP [Dem.] branché sur A6 c'est
     la durée de l'état "1" qui sera modifiée.
     
   NOTE : Certains choix d'E/S peuvent sembler étranges mais ils anticipent sur une répartition
          future qui envisage dès ce stade des études :
          • L'utilisation d'un BUZZER passif sur D11,
          • L'utilisation de trois LEDs sur A0, A1 et A2.
   
Branchements à effectuer :
//-------------------------- Entrées Analogiques -----------------------------
 A0 : LED Verte.
 A1 : LED Rouge.
 A2 : LED Bleue.
 A3 : Bouton central du codeur rotatif.
 A4 : SDA OLED.
 A5 : SCL OLED.
 A6 : BP_Dem.
//----------------------- Entrées / Sorties BINAIRES ------------------------- 
 Codeur rotatif B : Broche D2. >>>>>>>>>>> ATTENTION : En fonction du codeur KY-40 approvisionné il
 Codeur rotatif A : Broche D3. >>>>>>>>>>>>>>>>>>>>>>> faudra éventuellement inverser D2 et D3 pour
                               >>>>>>>>>>>>>>>>>> obtenir un sens de variation correct.
 D2  : A du Codeur rotatif.
 D3  : B du Codeur rotatif.
 D4  : Pilotage de la bobine B1.
 D5  : Pilotage de la bobine B2.
 D6  : Pilotage de la bobine B3.
 D7  : Pilotage de la bobine B4.
 D8  : Pilotage de la bobine B5.
 D9  : Pilotage de la bobine B6.
 D10  : Pilotage de la bobine B7.
 D12  : Pilotage de la bobine B8.
 D13  : Collisionneur.
 
                                                   PGM               RAM dynamique
Compilateur Version 1.7.9                 14030 Octets (45%)     330 Octets (16%)    */

//--------------------- Déclarations pour le matériel -----------------------
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)
//------------- Définition des broches d'entrées analogiques ----------------
#define LED_Verte 14
#define LED_Rouge 15
#define LED_Bleue 16
#define BP_central 17 // A3 pour lire le bouton central.
#define BP_Dem 20  // A6 pour lire le bouton Dem.

//-------------- Définition des broches de sorties Binaires -----------------
#define Codeur_B 2   // D2 : Codeur rotatif.
#define Codeur_A 3   // D3 : Codeur rotatif.
#define Bobine1 4    // D4 : Pilote B1.
#define Bobine2 5    // D5 : Pilote B2.
#define Bobine3 6    // D6 : Pilote B3.
#define Bobine4 7    // D7 : Pilote B4.
#define Bobine5 8    // D8 : Pilote B5.
#define Bobine6 9    // D9 : Pilote B6.
#define Bobine7 10   // D10 : Pilote B7.
#define Bobine8 12   // D12 : Pilote B8.
#define Collisionneur 13   // D12 : Pilote l'inducteur du collisionneur.

//-------------------------- Variables du programme -------------------------
boolean A_set, B_set;   // État des sorties déphasées du codeur.
boolean BP_central_a_traiter;
boolean INT_active;     // Indique un INT_active d'état détecté.
boolean Sens_positif; // Sens de rotation.

int Duree_zero, Duree_un; // Durées en Milli-seconde.

float Frequence, T0, T1;

unsigned long Temps_Ecoule_depuis_RESET = 0;

volatile boolean Rotation;  // Indique une activation du codeur rotatif.
//---------------------------------------------------------------------------

void setup() {
  //---------------------- Définition des vecteurs d'interruption ----------------------
  analogReference(DEFAULT);
  attachInterrupt(0, Front_sur_A, CHANGE); // Appel à Front_sur_A si INT_active d'état détecté.
  attachInterrupt(1, Front_sur_B, CHANGE); // Appel à Front_sur_B si INT_active d'état détecté.
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  pinMode(Codeur_A,INPUT); pinMode(Codeur_B,INPUT);
  pinMode(BP_Dem,INPUT_PULLUP); // Forcé au +5Vcc par résistance interne.
  pinMode(LED_Verte, OUTPUT); digitalWrite(LED_Verte, LOW);
  pinMode(LED_Rouge, OUTPUT); digitalWrite(LED_Rouge, LOW);
  pinMode(LED_Bleue, OUTPUT); digitalWrite(LED_Bleue, LOW);
  pinMode(Bobine1, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine2, OUTPUT); digitalWrite(Bobine2, LOW);
  pinMode(Bobine3, OUTPUT); digitalWrite(Bobine3, LOW);
  pinMode(Bobine4, OUTPUT); digitalWrite(Bobine4, LOW);
  pinMode(Bobine5, OUTPUT); digitalWrite(Bobine5, LOW);
  pinMode(Bobine6, OUTPUT); digitalWrite(Bobine6, LOW);
  pinMode(Bobine7, OUTPUT); digitalWrite(Bobine7, LOW);
  pinMode(Bobine8, OUTPUT); digitalWrite(Bobine8, LOW);
  pinMode(Bobine8, OUTPUT); digitalWrite(Collisionneur, LOW);
  //--------------------- Initialisation des variables ---------------------
  Duree_zero = 1; // Milli-secondes
  Duree_un = 2; // Milli-secondes.
  BP_central_a_traiter = false; Rotation = false;
  PAGE1(); Front_sur_A(); Front_sur_B();} // Voir informations en fin de listage.

void loop() {
  if (Rotation)TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL();
  //======================= Boucle de commutation =======================
  digitalWrite(Bobine1, HIGH); delay(Duree_un * 10); digitalWrite(Bobine1, LOW);
  digitalWrite(LED_Bleue, HIGH); delay(Duree_zero);  digitalWrite(LED_Bleue, LOW);
  
  digitalWrite(Bobine2, HIGH); delay(Duree_un * 10); digitalWrite(Bobine2, LOW);
  digitalWrite(LED_Verte, HIGH); delay(Duree_zero); digitalWrite(LED_Verte, LOW);
  
  digitalWrite(Bobine3, HIGH); delay(Duree_un * 10); digitalWrite(Bobine3, LOW);
  digitalWrite(LED_Rouge, HIGH); delay(Duree_zero);  digitalWrite(LED_Rouge, LOW);
  
  digitalWrite(Bobine4, HIGH); delay(Duree_un * 10); digitalWrite(Bobine4, LOW);
  digitalWrite(LED_Bleue, HIGH); delay(Duree_zero);  digitalWrite(LED_Bleue, LOW); 
  
  digitalWrite(Bobine5, HIGH); delay(Duree_un * 10); digitalWrite(Bobine5, LOW);
  digitalWrite(LED_Verte, HIGH); delay(Duree_zero);  digitalWrite(LED_Verte, LOW);
  
  digitalWrite(Bobine6, HIGH); delay(Duree_un * 10); digitalWrite(Bobine6, LOW);
  digitalWrite(LED_Rouge, HIGH); delay(Duree_zero);  digitalWrite(LED_Rouge, LOW);
  
  digitalWrite(Bobine7, HIGH); delay(Duree_un * 10); digitalWrite(Bobine7, LOW);
  digitalWrite(LED_Bleue, HIGH); delay(Duree_zero);  digitalWrite(LED_Bleue, LOW);
  
  digitalWrite(Bobine8, HIGH); delay(Duree_un * 10); digitalWrite(Bobine8, LOW);
  digitalWrite(LED_Verte, HIGH); delay(Duree_zero); digitalWrite(LED_Verte, LOW);}

//-------------------- Routines de servitude pour traiter le capteur incrémental -------------------
  void Delay_antirebonds() {for(int T=0; T<1000; T++);} // Valeur de 100 expérimentale.
  
  void Front_sur_A(){ // Interruption quand A change d'état avant B.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds.
    if (digitalRead(Codeur_A) != A_set ) {// Test pour savoir si l'état a changé.
      A_set = !A_set;
      if (A_set && !B_set) {Sens_positif = true; Rotation = true; Delay_antirebonds();} } }
  
  void Front_sur_B(){ // Interruption quand B change d'état avant A.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds. 
    if( digitalRead(Codeur_B) != B_set ) {// Test pour savoir si l'état a changé.
      B_set = !B_set;
      // Décrémenter le compteur si B avant A.
      if (B_set && !A_set) {Sens_positif = false; Rotation = true; Delay_antirebonds();} } }
  //--------------------------------------------------------------------------------------------------
  
void PAGE1() {
  T0 = (float(Duree_zero ))/ 1000; T1 = (float(Duree_un)) / 1000;
  Frequence = (1000 / float(Duree_zero + Duree_un));
  u8g.setFont(u8g_font_6x10); // Pocice de matrices de 6x10 pixels lettres MJA/min comprises.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //================ Définition des éléments à afficher ================
  CADRES();
  Aff_TEXTE(15, 9 ,"Sorties D4 a D12.");
  Aff_TEXTE(25,26,"Etat 1 : "); u8g.print(T1,3); u8g.print(" s");
  Aff_TEXTE(25,41,"Etat 0 : "); u8g.print(T0,3); u8g.print(" s");
  Aff_TEXTE(6,55,"Frequence : "); u8g.print(Frequence,2); u8g.print(" Hz");}
  //====================================================================
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
 
void CADRES() {u8g.drawFrame(0,0,128,13); u8g.drawFrame(0,15,128,48);} // Cadre TITRE et cadre données.
  
void Aff_TEXTE(byte X, byte Y, char TEXTE[]) {
  char Caractere; byte I = 0;
  while (Caractere !=0) {Caractere = TEXTE[I]; u8g.setPrintPos((I*6)+X, Y); I++; u8g.print(Caractere);}}

void Teste_activation_BP_central() {
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
     // ========== Attendre le relacher du B.P. central. ==========
     while (analogRead(BP_central) < 512); // Attendre le relâché.
     // ===========================================================
     BP_central_a_traiter = true;}}

void TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL() {
  digitalWrite(Collisionneur, HIGH);
  while (!BP_central_a_traiter) {
        if (Rotation) {
           if (analogRead(BP_Dem) < 512) {
              if (Sens_positif) {if (Duree_un < 500) Duree_un = Duree_un + 1;}
              else {if (Duree_un > 1) Duree_un = Duree_un - 1;}}    
           else {if (Sens_positif) {if (Duree_zero < 300) Duree_zero = Duree_zero + 1;}
              else {if (Duree_zero != 0)Duree_zero = Duree_zero - 1;}}
        PAGE1(); Rotation = false; INT_active = true;}
        // Ci-dessus : Armer un nouveau comptage et Prise en compte du traitement.
        Teste_activation_BP_central();}
        BP_central_a_traiter = false; digitalWrite(Collisionneur, LOW);}

/* Problèmes liés au codeur rotatif :
   ----------------------------------
   Ce type de codeur engendre une rotation fantôme positive et négative à sa première utilisation.
   Du coup sur un RESET le programme est en mode saisie des durées. 
  
   En fonction du codeur rotatif approvisionné, il peut s'avérer nécessaire d'inverser D2 et D3
   pour obtenir le sens de déplacement correct.

   Ayant approvisionné ce type de codeur chez divers fournisseurs, les tarifs ne sont pas les mêmes,
   mais la qualité non plus. Les moins onéreux sont victimes de faux contacts qui se traduisent par
   des comptages aléatoires. Aussi, se c'est le cas pour le composant utilisé chez vous, un moyen
   radical d'éliminer ce problème particulièrement pénalisant consiste à augmenter "la constante de
   tems" de l'établissement du niveau électrique logique en plaçant un condendateur entre A, B et GND.
   Des valeurs de l'ordre de 10nF conviennent sans problème. */


