/* Programme complet sauf les séquences pour l'EXPLOITATION : Strictement identique au programme
   P07 mais pour lequel les textes affichés sur OLED sont logés en EEPROM dans le but
   d'optimiser la taille du code généré par le compilateur.

   Programme qui met en place le MENU de BASE et les aiguillages pour les items.
   Si Clic central valide l'item indexé.
   Sortie de l'item avec le B.P. central et retour au MENU de BASE.
   Tant que le B.P. central est appuyé, la LED d'arduino s'illumine.
   Si on valide EXPLOITATION l'écran affiche cette fonction et le logiciel
   devient inerte, il faut un RESET pour éventuellement retourner au MENU.
   • Sur RESET il y a passage à l'EXPLOITATION.
   • Sur RESET et BP Dem; activé, il y a ouverture du MENU de BASE.
   
   >>>> Lire les commentaires sur ce programme en fin de listage.

Branchements à effectuer :
--------------------------
 A0 : LED Verte.
 A1 : LED Rouge.
 A2 : LED Bleue.
 A3 : Bouton central du codeur rotatif.
 A4 : SDA OLED.
 A5 : SCL OLED.
 Codeur rotatif B : Broche D2. >>>>>>>>>>> ATTENTION : En fonction du codeur KY-40 approvisionné il
 Codeur rotatif A : Broche D3. >>>>>>>>>>>>>>>>>>>>>>> faudra éventuellement inverser D2 et D3 pour
                               >>>>>>>>>>>>>>>>>> obtenir un sens de variation correct.
 D2  : A du Codeur rotatif.
 D3  : B du Codeur rotatif.
 D4  : Pilotage de la bobine B1.
 D5  : Pilotage de la bobine B2.
 D6  : Pilotage de la bobine B3.
 D7  : Pilotage de la bobine B4.
 D8  : Pilotage de la bobine B5.
 D9  : Pilotage de la bobine B6.
 D10  : Pilotage de la bobine B7.
 D12  : Pilotage de la bobine B8.
 D13  : Collisionneur.
 
Compilateur Version 1.7.9                         PGM               RAM dynamique
Textes en mémoire programme :              12412 Octets (40%)     426 Octets (20%)
Textes en mémoire EEPROM :                 12302 Octets (40%)     284 Octets (13%) */

//--------------------- Déclarations pour le matériel -----------------------
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)

//------------- Définition des broches d'entrées analogiques ----------------
#define BP_central 17 // A3 pour lire le bouton central.
#define BP_Marche 20 // A6 pour lire le BP Marche.
#define BP_Tir 21 // A7 pour lire le BP Tir.

//------------- Définition des broches de sorties analogiques ---------------
#define LED_Clavier 13 // LED rouge d'Arduino.
#define VERTE 14 // A0 pour piloter la LED Verte.
#define ROUGE 15 // A1 pour piloter la LED Rouge.
#define BLEUE 16 // A2 pour piloter la LED Bleue.
//-------------- Définition des broches de sorties Binaires -----------------
#define Codeur_A 2    // D2 : DT  >>>>>>>>>>>>>>>>>>> ATTENTION : En fonction du codeur KY-40 approvisionné il
#define Codeur_B 3    // D3 : CLK >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> faudra éventuellement inverser D2 et D3 pour
#define Bobine1 4    // D4 : Pilote B1.
#define Bobine2 5    // D5 : Pilote B2.
#define Bobine3 6    // D6 : Pilote B3.
#define Bobine4 7    // D7 : Pilote B4.
#define Bobine5 8    // D8 : Pilote B5.
#define Bobine6 9    // D9 : Pilote B6.
#define Bobine7 10   // D10 : Pilote B7.
#define Buzzer  11   // D11 : Bruiteur passif.
#define Bobine8 12   // D12 : Pilote B8.
#define TIR 13       // D13 : Pilote le collisionneur.

#define Version_PGM "06/06/2023"  //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

//----------------------- Déclaration des variables -------------------------
boolean BP_central_a_traiter;
boolean A_set, B_set;   // État des sorties déphasées du codeur.
boolean INT_active;     // Indique un INT_active d'état détecté.
boolean Sens_positif; // Sens de rotation.
boolean Effacer_EEPROM;

boolean FINI;

byte Position_Index;

int Nb_TIRs, Nb_Cycles;

volatile boolean Rotation;  // Indique une activation du codeur rotatif.

//---------------------------------------------------------------------------

void setup() {
  //---------------------- Définition des vecteurs d'interruption ----------------------
  attachInterrupt(0, Front_sur_A, CHANGE); // Appel à Front_sur_A si INT_active d'état détecté.
  // Broche de l'interruption 1 (E/S D3) :
  attachInterrupt(1, Front_sur_B, CHANGE); // Appel à Front_sur_B si INT_active d'état détecté.
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  pinMode(Codeur_A,INPUT); pinMode(Codeur_B,INPUT);
  pinMode(VERTE, OUTPUT); pinMode(ROUGE, OUTPUT); pinMode(BLEUE, OUTPUT); Eteindre_LEDs();
  pinMode(Bobine1, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine2, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine3, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine4, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine5, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine6, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine7, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine8, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(TIR, OUTPUT); digitalWrite(TIR, LOW);
  BP_central_a_traiter = false; digitalWrite(BP_central, HIGH); // Valide R PUL-UP sur A2.
  Nb_TIRs = eeprom_read_word((unsigned int *) 1020);
  Nb_Cycles = eeprom_read_word((unsigned int *) 1022);
  Position_Index = 3; Front_sur_A(); Front_sur_B(); // Voir informations en fin de listage.
  if (analogRead(BP_Marche) < 512) MENU_de_BASE();}

void loop() {EXPLOITATION();}

  //-------------------- Routines de servitude pour traiter le capteur incrémental -------------------
  void Delay_antirebonds() {for(int T=0; T<1000; T++);} // Valeur de 100 expérimentale.
  
  void Front_sur_A(){ // Interruption quand A change d'état avant B.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds.
    if (digitalRead(Codeur_A) != A_set ) {// Test pour savoir si l'état a changé.
      A_set = !A_set;
      if (A_set && !B_set) {Sens_positif = true; Rotation = true; Delay_antirebonds();} } }
  
  void Front_sur_B(){ // Interruption quand B change d'état avant A.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds. 
    if( digitalRead(Codeur_B) != B_set ) {// Test pour savoir si l'état a changé.
      B_set = !B_set;
      // Décrémenter le compteur si B avant A.
      if (B_set && !A_set) {Sens_positif = false; Rotation = true; Delay_antirebonds();} } }
      
  //--------------------------------------------------------------------------------------------------

void Eteindre_LEDs() {
  digitalWrite(VERTE, LOW); digitalWrite(ROUGE, LOW); digitalWrite(BLEUE, LOW);}

void BIP() {tone(Buzzer, 3000, 150);}
  // La déclaration en sortie de BUZZER n'est pas nécessaire si usage de l'instruction tone.

void Aff_TEXTE_EEPROM(int PTR, byte Longueur) {// Routines pour les affichages textes EEPROM.
    for (byte I=0; I<Longueur; I++) {u8g.print(char(eeprom_read_byte((unsigned char *)PTR))); PTR++;}; }

void TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL() {
  INT_active = true; // Armer un nouveau comptage.
  Rotation = false; // Prise en compte du traitement.
  if (Sens_positif) {Position_Index++; if (Position_Index == (4)) Position_Index = 0;}
  else {if (Position_Index > 0) Position_Index--; else Position_Index = 3;}}

void Teste_activation_BP_central() {
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
     // ========== Attendre le relacher du B.P. central. ==========
     digitalWrite(LED_Clavier, HIGH); // Allume la LED.
     while (analogRead(BP_central) < 512); // Attendre le relâché.
     digitalWrite(LED_Clavier, LOW);
     // ===========================================================
     BP_central_a_traiter = true;}}

void Traite_BP_central() {
  Teste_activation_BP_central(); if (BP_central_a_traiter) {FINI = true;
  BP_central_a_traiter = false; Position_Index = 0;}}

void RAZ() {
  FINI = false; Effacer_EEPROM = false;
  while (!FINI) {
    //---------------------- Affichage pour la RAZ ---------------------
    u8g.firstPage(); do {
      u8g.setPrintPos(30,16); Aff_TEXTE_EEPROM(0, 11); // "Effacer les"
      u8g.setPrintPos(40,28); Aff_TEXTE_EEPROM(11, 7); // "valeurs"
      u8g.setPrintPos(30,40); Aff_TEXTE_EEPROM(18, 11); // "en EEPROM :"
      u8g.setPrintPos(35,53); Aff_TEXTE_EEPROM(29, 4);
      if (Effacer_EEPROM) Aff_TEXTE_EEPROM(33, 3); else Aff_TEXTE_EEPROM(36, 3);}
      while(u8g.nextPage());
    //-------------- Traite la rotation du codeur incrémental ----------
    if (Rotation) {Effacer_EEPROM = !Effacer_EEPROM; Rotation = false;}
    //--------------------- Traite le B.P. central ---------------------
    Teste_activation_BP_central();
    if (BP_central_a_traiter) {
       BP_central_a_traiter = false; FINI = true;
       //---------------- Efface les donnees en EEPROM -----------------
       if (Effacer_EEPROM) {
          eeprom_write_word((unsigned int *) 1020, 0); Nb_TIRs = 0;
          eeprom_write_word((unsigned int *) 1022, 0); Nb_Cycles = 0;}}}
    FINI=false;} // Pour rester dans le MENU_de_BASE.

void Impulsion(byte Sortie) {
  digitalWrite(BLEUE, HIGH);
  digitalWrite(Sortie, HIGH); delay(2000); digitalWrite(Sortie, LOW);
  Eteindre_LEDs();}
  
void Mode_MANUEL() {
  byte Sortie; byte Inducteur_en_test = 1;
  FINI = false;
  while (!FINI) {
    //------------------ Afficher l'écran de MANUEL : -----------------
    u8g.firstPage(); do {
      u8g.drawFrame(61,38,11,12);
      u8g.setPrintPos(40,16); Aff_TEXTE_EEPROM(39, 11); // "Mode MANUEL"
      u8g.setPrintPos(10,32);  Aff_TEXTE_EEPROM(50, 18); // "Inducteur a tester"
      u8g.setPrintPos(64,48); if (Inducteur_en_test != 9) u8g.print(Inducteur_en_test); else u8g.print('C');}
    while(u8g.nextPage());
  //------------------ Traite les deux boutons poussoir ----------------
  if (analogRead(BP_Marche) < 512) {
     switch (Inducteur_en_test) {
       case 1 : {Sortie = Bobine1; Impulsion(Sortie); break;}
       case 2 : {Sortie = Bobine2; Impulsion(Sortie); break;}
       case 3 : {Sortie = Bobine3; Impulsion(Sortie); break;}
       case 4 : {Sortie = Bobine4; Impulsion(Sortie); break;}
       case 5 : {Sortie = Bobine5; Impulsion(Sortie); break;}
       case 6 : {Sortie = Bobine6; Impulsion(Sortie); break;}
       case 7 : {Sortie = Bobine7; Impulsion(Sortie); break;}
       case 8 : {Sortie = Bobine8; Impulsion(Sortie); break;}
       case 9 : {digitalWrite(ROUGE, HIGH); BIP(); Eteindre_LEDs();}}
       while (analogRead(BP_Marche) < 512);} // Attendre le relâcher.
  if (analogRead(BP_Tir) < 512) {
      digitalWrite(VERTE, HIGH);
      digitalWrite(TIR, HIGH); delay(2000); digitalWrite(TIR, LOW); Eteindre_LEDs();
      while (analogRead(BP_Tir) < 512);} // Attendre le relâcher.
  //--------------- Traite la rotation du codeur incrémental -----------
  if (Rotation) {Rotation = false; // Prise en compte du traitement.
  if (!Sens_positif) {Inducteur_en_test++; if (Inducteur_en_test == 10) Inducteur_en_test = 1;}
  else {if (Inducteur_en_test > 1) Inducteur_en_test--; else Inducteur_en_test = 9;}}
  //---------------------- Traite le B.P. central ----------------------
  Teste_activation_BP_central();
  if (BP_central_a_traiter) {BP_central_a_traiter = false; FINI = true;}}
  FINI=false;} // Pour rester dans le MENU_de_BASE.}

void MENU_STATISTIQUES() {
  FINI = false;
  u8g.firstPage(); do {
    u8g.setPrintPos(22,26); Aff_TEXTE_EEPROM(70, 12); u8g.print(Nb_Cycles);// "Nb Cycles = "
    u8g.setPrintPos(34,38); Aff_TEXTE_EEPROM(82, 10); u8g.print(Nb_TIRs); // "Nb TIRs = "
    u8g.setPrintPos(3,53); Aff_TEXTE_EEPROM(92, 11); u8g.print(Version_PGM);}
    while(u8g.nextPage());
  while (!FINI) {Traite_BP_central();
        if (BP_central_a_traiter) {BP_central_a_traiter = false; FINI = true;}}
  FINI=false;} // Pour rester dans le MENU_de_BASE.

void MENU_de_BASE() {
  FINI = false;
  while(!FINI) {
    //-------------------- Afficher le MENU de BASE : --------------------
    u8g.firstPage(); do {
      u8g.setPrintPos(40,16); Aff_TEXTE_EEPROM(103, 12); // "EXPLOITATION"
      u8g.setPrintPos(40,28); Aff_TEXTE_EEPROM(115, 12); // Statistiques"
      u8g.setPrintPos(40,40); Aff_TEXTE_EEPROM(127, 11); // Mode MANUEL"
      u8g.setPrintPos(40,53); Aff_TEXTE_EEPROM(138, 3); // RAZ"
      u8g.setPrintPos(30,16 + (Position_Index * 12)); u8g.print('>');}
      while(u8g.nextPage());
    //--------------- Traite la rotation du codeur incrémental -----------
    if (Rotation) TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL();
    //---------------------- Traite le B.P. central ----------------------
    Teste_activation_BP_central();
    if (BP_central_a_traiter) {
       BP_central_a_traiter = false;
    switch (Position_Index) {
       case 0 : {FINI = true; break;}
       case 1 : {MENU_STATISTIQUES(); break;}
       case 2 : {Mode_MANUEL(); break;}
       case 3 : RAZ();}}}}

void EXPLOITATION() {
  u8g.firstPage();
  do {u8g.setPrintPos(30,35); Aff_TEXTE_EEPROM(103, 12);} while(u8g.nextPage());}
  
/* Problèmes liés au codeur rotatif :
   ----------------------------------
   Ce type de codeur engendre une rotation fantôme positive et négative à sa première utilisation.
   Pour parer cette difficulté, les instructions suivantes sont ajoutées dans les initialisations :
   Front_sur_A(); Front_sur_B(); Num_PGM_indexee = 4;
   Cette ligne de code effectue une "rotation positive artificielle". Du coup Num_PGM_indexee prend
   la valeur 0 en tête de listage et le fonctionnement du codeur rotatif est correct. 
  
   En fonction du codeur rotatif approvisionné, il peut s'avérer nécessaire d'inverser D2 et D3
   pour obtenir le sens de déplacement correct. */
  
  
