/* Programme complet d'exploitation de la machine.

   • Sur RESET : Ouverture du menu des commandes si BP Dem activé.
                 Ouverture de l'écran "Kilométrage" si BP TIR activé.
                 Ouverture directe du MENU de BASE si BP Dem NON activé.
   • Dans l'attente d'un cycle Le BP central du codeur rotatif inverse SILENCE.
   • Dans l'attente d'un cycle le codeur rotatif change les pages-écran affichées.
     Si l'écran est noir les LEDs sont aussi éteintes.
   • La consommation en énergie électrique de la machine n'est prise en compte
     que durant le mode EXPLOITATION.
   
   >>>> Lire les commentaires sur ce programme en fin de listage.

Branchements à effectuer :
--------------------------
 A0 : LED Verte.
 A1 : LED Rouge.
 A2 : LED Bleue.
 A3 : Bouton central du codeur rotatif.
 A4 : SDA OLED.
 A5 : SCL OLED.
 Codeur rotatif B : Broche D2. >>>>>>>>>>> ATTENTION : En fonction du codeur KY-40 approvisionné il
 Codeur rotatif A : Broche D3. >>>>>>>>>>>>>>>>>>>>>>> faudra éventuellement inverser D2 et D3 pour
                               >>>>>>>>>>>>>>>>>> obtenir un sens de variation correct.
 D2  : A du Codeur rotatif.
 D3  : B du Codeur rotatif.
 D4  : Pilotage de la bobine B1.
 D5  : Pilotage de la bobine B2.
 D6  : Pilotage de la bobine B3.
 D7  : Pilotage de la bobine B4.
 D8  : Pilotage de la bobine B5.
 D9  : Pilotage de la bobine B6.
 D10  : Pilotage de la bobine B7.
 D12  : Pilotage de la bobine B8.
 D13  : Collisionneur.
                                                   PGM               RAM dynamique
Compilateur Version 1.7.9                    18602 Octets (60%)     349 Octets (17%) */

//--------------------- Déclarations pour le matériel -----------------------
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)

//------------- Définition des broches d'entrées analogiques ----------------
#define BP_central 17 // A3 pour lire le bouton central.
#define BP_Marche 20 // A6 pour lire le BP Marche.
#define BP_Tir 21 // A7 pour lire le BP Tir.

//------------- Définition des broches de sorties analogiques ---------------
#define LED_Clavier 13 // LED rouge d'Arduino. ATTENTION : Même sortie pour TIR.
#define VERTE 14 // A0 pour piloter la LED Verte.
#define ROUGE 15 // A1 pour piloter la LED Rouge.
#define BLEUE 16 // A2 pour piloter la LED Bleue.
//-------------- Définition des broches de sorties Binaires -----------------
#define Codeur_A 2    // D2 : DT  >>>>>>>>>>>>>>>>>>> ATTENTION : En fonction du codeur KY-40 approvisionné il
#define Codeur_B 3    // D3 : CLK >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> faudra éventuellement inverser D2 et D3 pour
#define Bobine1 4    // D4 : Pilote B1.
#define Bobine2 5    // D5 : Pilote B2.
#define Bobine3 6    // D6 : Pilote B3.
#define Bobine4 7    // D7 : Pilote B4.
#define Bobine5 8    // D8 : Pilote B5.
#define Bobine6 9    // D9 : Pilote B6.
#define Bobine7 10   // D10 : Pilote B7.
#define Buzzer  11   // D11 : Bruiteur passif.
#define Bobine8 12   // D12 : Pilote B8.
#define TIR 13       // D13 : Pilote le collisionneur.

#define Version_PGM "15/06/2023"  //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

//----------------------- Déclaration des variables -------------------------

boolean A_set, B_set;   // État des sorties déphasées du codeur.
boolean BP_central_a_traiter;
boolean Cycle_en_cours;
boolean INT_active;     // Indique un INT_active d'état détecté.
boolean Sens_positif; // Sens de rotation.
boolean Silence;
boolean Effacer_EEPROM;
boolean Mode_EXPLOITATION;

boolean FINI, Refroidissement, TIR_valide;

byte Index, Position_Index, Temperature;

int Duree_un; // Durées en centièmes de seconde.
int Frequence;
int Nb_TIRs, Nb_Cycles, Tir_depuis_RESET, Cycles_depuis_RESET;

long NB_Ecritures_EEPROM;

unsigned long Temps_Ecoule_depuis_RESET = 0;

float Energie, Energie_totale;

volatile boolean Rotation;  // Indique une activation du codeur rotatif.

//---------------------------------------------------------------------------

void setup() {
  //---------------------- Définition des vecteurs d'interruption ----------------------
  attachInterrupt(0, Front_sur_A, CHANGE); // Appel à Front_sur_A si INT_active d'état détecté.
  // Broche de l'interruption 1 (E/S D3) :
  attachInterrupt(1, Front_sur_B, CHANGE); // Appel à Front_sur_B si INT_active d'état détecté.
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  pinMode(Codeur_A,INPUT); pinMode(Codeur_B,INPUT);
  pinMode(VERTE, OUTPUT); pinMode(ROUGE, OUTPUT); pinMode(BLEUE, OUTPUT); Eteindre_LEDs();
  pinMode(Bobine1, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine2, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine3, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine4, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine5, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine6, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine7, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine8, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(TIR, OUTPUT); digitalWrite(TIR, LOW);
  BP_central_a_traiter = false; digitalWrite(BP_central, HIGH); // Valide R PUL-UP sur A2.
  Cycle_en_cours = false;
  Nb_TIRs = eeprom_read_word((unsigned int *) 1020);
  Nb_Cycles = eeprom_read_word((unsigned int *) 1022);
  Energie = 0.0; Energie_totale = eeprom_read_float((float *)1016);
  NB_Ecritures_EEPROM = eeprom_read_dword((unsigned long *) 1012);
  Index = 0; // Menu dans EXPLOITATION.
  TIR_valide = false; Temperature = 1; Tir_depuis_RESET = 0; Cycles_depuis_RESET = 0;
  Refroidissement = false; Silence = false; Mode_EXPLOITATION = false;
  Position_Index = 3; Front_sur_A(); Front_sur_B(); // Voir informations en fin de listage.
  //---------------------- Traite le BP DEM. ---------------------
  if (analogRead(BP_Marche) < 512) {
      while (analogRead(BP_Marche) < 512); // Attendre le relâcher.
      MENU_des_commandes();}
  //---------------------- Traite le BP TIR. ---------------------
  if (analogRead(BP_Tir) < 512) {
      while (analogRead(BP_Tir) < 512); // Attendre le relâcher.
      Page_kilometrique();}
  MENU_de_BASE();}

void loop() {EXPLOITATION();}

  //-------------------- Routines de servitude pour traiter le capteur incrémental -------------------
  void Delay_antirebonds() {for(int T=0; T<1000; T++);} // Valeur de 100 expérimentale.
  
  void Front_sur_A(){ // Interruption quand A change d'état avant B.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds.
    if (digitalRead(Codeur_A) != A_set ) {// Test pour savoir si l'état a changé.
      A_set = !A_set;
      if (A_set && !B_set) {Sens_positif = true; Rotation = true; Delay_antirebonds();}}}
  
  void Front_sur_B(){ // Interruption quand B change d'état avant A.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds. 
    if( digitalRead(Codeur_B) != B_set ) {// Test pour savoir si l'état a changé.
      B_set = !B_set;
      // Décrémenter le compteur si B avant A.
      if (B_set && !A_set) {Sens_positif = false; Rotation = true; Delay_antirebonds();} } }
      
  //--------------------------------------------------------------------------------------------------

void Eteindre_LEDs() {
  digitalWrite(VERTE, LOW); digitalWrite(ROUGE, LOW); digitalWrite(BLEUE, LOW);}

void BIP() {tone(Buzzer, 3000, 150);}
  // La déclaration en sortie de BUZZER n'est pas nécessaire si usage de l'instruction tone.

void Aff_TEXTE_EEPROM(int PTR, byte Longueur) {// Routines pour les affichages textes EEPROM.
    for (byte I=0; I<Longueur; I++) {u8g.print(char(eeprom_read_byte((unsigned char *)PTR))); PTR++;}; }

void TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL() {
  INT_active = true; // Armer un nouveau comptage.
  Rotation = false; // Prise en compte du traitement.
  if (Sens_positif) {Position_Index++; if (Position_Index == (4)) Position_Index = 0;}
  else {if (Position_Index > 0) Position_Index--; else Position_Index = 3;}}

void Teste_activation_BP_central() {
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
     // ========== Attendre le relacher du B.P. central. ==========
     if (!Mode_EXPLOITATION) digitalWrite(LED_Clavier, HIGH); // Allume la LED.
     while (analogRead(BP_central) < 512); // Attendre le relâché.
     digitalWrite(LED_Clavier, LOW);
     // ===========================================================
     BP_central_a_traiter = true;}}

void Traite_BP_central() {
  Teste_activation_BP_central(); if (BP_central_a_traiter) {FINI = true;
  BP_central_a_traiter = false; Position_Index = 0;}}

void OUI_NON(boolean OUI) {
  if (OUI) Aff_TEXTE_EEPROM(33, 3); else Aff_TEXTE_EEPROM(36, 3);}

void RAZ() {
  FINI = false; Effacer_EEPROM = false;
  while (!FINI) {
    //---------------------- Affichage pour la RAZ ---------------------
    u8g.firstPage(); do {
      u8g.drawFrame(0,0,128,64); u8g.drawFrame(2,2,124,60);
      u8g.setPrintPos(30,16); Aff_TEXTE_EEPROM(0, 11); // "Effacer les"
      u8g.setPrintPos(40,28); Aff_TEXTE_EEPROM(11, 7); // "valeurs"
      u8g.setPrintPos(30,40); Aff_TEXTE_EEPROM(18, 11); // "en EEPROM :"
      u8g.setPrintPos(35,53); Aff_TEXTE_EEPROM(29, 4);
      OUI_NON(Effacer_EEPROM);}
      while(u8g.nextPage());
    //-------------- Traite la rotation du codeur incrémental ----------
    if (Rotation) {Effacer_EEPROM = !Effacer_EEPROM; Rotation = false;}
    //--------------------- Traite le B.P. central ---------------------
    Teste_activation_BP_central();
    if (BP_central_a_traiter) {
       BP_central_a_traiter = false; FINI = true;
       //---------------- Efface les donnees en EEPROM -----------------
       if (Effacer_EEPROM) {
          eeprom_write_word((unsigned int *) 1020, 0); Nb_TIRs = 0;
          eeprom_write_word((unsigned int *) 1022, 0); Nb_Cycles = 0;
          eeprom_write_float((float *)1016,0.0);}}}
    FINI=false;} // Pour rester dans le MENU_de_BASE.

void Impulsion(byte Sortie) {
  digitalWrite(BLEUE, HIGH);
  digitalWrite(Sortie, HIGH); delay(2000); digitalWrite(Sortie, LOW);
  Eteindre_LEDs();}

void Incremente_Energie_toutes_les_trois_secondes() {
  //================= Chronométrage pour le délai écoulé =============================
  if (millis() - Temps_Ecoule_depuis_RESET > 3000)  {
     Temps_Ecoule_depuis_RESET = millis(); // réarme un prochain comptage.
     //----------------- Actions à traiter pour chaque délai écoulé. -----------------
     Energie = Energie + 0.0011; Energie_totale = (eeprom_read_float((float *)1016)) + 0.0011;
     eeprom_write_float((float *)1016,Energie_totale); NB_Ecritures_EEPROM++;
     eeprom_write_dword((unsigned long *) 1012,NB_Ecritures_EEPROM);}}
  
void Mode_MANUEL() {
  byte Sortie; byte Inducteur_en_test = 1;
  FINI = false;
  while (!FINI) {
    //================= Chronométrage pour le délai écoulé =============================
    Incremente_Energie_toutes_les_trois_secondes();
    //------------------ Afficher l'écran de MANUEL : -----------------
    u8g.firstPage(); do {
      u8g.drawFrame(0,0,128,64); u8g.drawFrame(2,2,124,60); u8g.drawHLine(3,22,122);
      u8g.setPrintPos(34,15); Aff_TEXTE_EEPROM(39, 11); // "Mode MANUEL"
      u8g.setPrintPos(10,36);  Aff_TEXTE_EEPROM(50, 18); // "Inducteur a tester"
      u8g.drawCircle(40,49,8); u8g.drawCircle(95,49,8);
      if (Inducteur_en_test != 9) {u8g.setPrintPos(38,53); u8g.print(Inducteur_en_test);}
      else {u8g.setPrintPos(93,53); u8g.print('C');}}
    while(u8g.nextPage());
  //------------------ Traite les deux boutons poussoir ----------------
  if (analogRead(BP_Marche) < 512) {
     switch (Inducteur_en_test) {
       case 1 : {Sortie = Bobine1; Impulsion(Sortie); break;}
       case 2 : {Sortie = Bobine2; Impulsion(Sortie); break;}
       case 3 : {Sortie = Bobine3; Impulsion(Sortie); break;}
       case 4 : {Sortie = Bobine4; Impulsion(Sortie); break;}
       case 5 : {Sortie = Bobine5; Impulsion(Sortie); break;}
       case 6 : {Sortie = Bobine6; Impulsion(Sortie); break;}
       case 7 : {Sortie = Bobine7; Impulsion(Sortie); break;}
       case 8 : {Sortie = Bobine8; Impulsion(Sortie); break;}
       case 9 : {digitalWrite(ROUGE, HIGH); BIP(); Eteindre_LEDs();}}
       while (analogRead(BP_Marche) < 512);} // Attendre le relâcher.
  if (analogRead(BP_Tir) < 512) {
      if (Inducteur_en_test == 9) {
         digitalWrite(VERTE, HIGH);
         digitalWrite(TIR, HIGH); delay(2000); digitalWrite(TIR, LOW); Eteindre_LEDs();}
      else BIP();
      while (analogRead(BP_Tir) < 512);} // Attendre le relâcher.
  //--------------- Traite la rotation du codeur incrémental -----------
  if (Rotation) {Rotation = false; // Prise en compte du traitement.
  if (!Sens_positif) {Inducteur_en_test++; if (Inducteur_en_test == 10) Inducteur_en_test = 1;}
  else {if (Inducteur_en_test > 1) Inducteur_en_test--; else Inducteur_en_test = 9;}}
  //---------------------- Traite le B.P. central ----------------------
  Teste_activation_BP_central();
  if (BP_central_a_traiter) {BP_central_a_traiter = false; FINI = true;}}
  FINI=false;} // Pour rester dans le MENU_de_BASE.}

void MENU_STATISTIQUES() {
  u8g.firstPage(); do {
    u8g.drawFrame(0,0,128,64); u8g.drawFrame(2,2,124,60);
    u8g.setPrintPos(22,17); Aff_TEXTE_EEPROM(70, 12); u8g.print(Nb_Cycles);// "Nb Cycles = "
    u8g.setPrintPos(34,29); Aff_TEXTE_EEPROM(82, 10); u8g.print(Nb_TIRs); // "Nb TIRs = "
    u8g.setPrintPos(21,41); u8g.print("PW : ");
    u8g.print(eeprom_read_float((float *)1016),4); u8g.print(" Wh");
    u8g.setPrintPos(15,54);  u8g.print("PGM du ");; u8g.print(Version_PGM);}
    while(u8g.nextPage());
  FINI = false;
  while (!FINI) {Traite_BP_central();
        if (BP_central_a_traiter) {BP_central_a_traiter = false; FINI = true;}}
  Position_Index = 1; FINI=false;} // Pour rester dans le MENU_de_BASE.

void MENU_de_BASE() {
  FINI = false;
  while(!FINI) {
    //================= Chronométrage pour le délai écoulé =============================
    Incremente_Energie_toutes_les_trois_secondes();
    //==================================================================================
    //-------------------- Afficher le MENU de BASE : --------------------
    u8g.firstPage(); do {
      u8g.drawFrame(0,0,128,64); u8g.drawFrame(2,2,124,60);
      u8g.setPrintPos(34,17); Aff_TEXTE_EEPROM(103, 12); // "EXPLOITATION"
      u8g.setPrintPos(34,29); Aff_TEXTE_EEPROM(115, 12); // Statistiques"
      u8g.setPrintPos(34,41); Aff_TEXTE_EEPROM(127, 11); // Mode MANUEL"
      u8g.setPrintPos(34,54); Aff_TEXTE_EEPROM(138, 3); // RAZ"
      u8g.setPrintPos(25,17 + (Position_Index * 12)); u8g.print('>');}
      while(u8g.nextPage());
    //--------------- Traite la rotation du codeur incrémental -----------
    if (Rotation) TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL();
    //---------------------- Traite le B.P. central ----------------------
    Teste_activation_BP_central();
    if (BP_central_a_traiter) {
       BP_central_a_traiter = false;
       switch (Position_Index) {
         case 0 : {FINI = true; Index = 1; break;}
         case 1 : {MENU_STATISTIQUES(); break;}
         case 2 : {Mode_MANUEL(); break;}
         case 3 : RAZ();}}}}

void Ecran_1() {
  u8g.firstPage(); do {
  u8g.setPrintPos(30,8); Aff_TEXTE_EEPROM(208, 9); // "Ce MENU :"
  u8g.drawHLine(30,10,52);
  u8g.setPrintPos(0,23); Aff_TEXTE_EEPROM(217, 21); // "BPC : Sortie du mode."
  u8g.setPrintPos(0,36); Aff_TEXTE_EEPROM(238, 21); // "Rot : Change de page."
  u8g.setPrintPos(0,49); Aff_TEXTE_EEPROM(259, 12); // "Tir : Ignoré"
  u8g.setPrintPos(0,62); Aff_TEXTE_EEPROM(439, 12);} // "Dem : Ignoré"
  while(u8g.nextPage());}

void Ecran_2() {
  u8g.firstPage(); do {
  u8g.setPrintPos(18,8); Aff_TEXTE_EEPROM(271, 14); // "EXPLOITATION :"
  u8g.drawHLine(18,10,82);
  u8g.setPrintPos(0,23); Aff_TEXTE_EEPROM(285, 19); // "BPC : Mode SILENCE."
  u8g.setPrintPos(0,36); Aff_TEXTE_EEPROM(304, 18); // "Rot : Pages écran."
  u8g.setPrintPos(0,49); Aff_TEXTE_EEPROM(322, 20); // "Tir : TIR > OUI NON."
  u8g.setPrintPos(0,62); Aff_TEXTE_EEPROM(342, 19);} // "Dem : Débute cycle."
  while(u8g.nextPage());}

void Ecran_3() {
  u8g.firstPage(); do {
  u8g.setPrintPos(12,8); Aff_TEXTE_EEPROM(361, 14); // "MENU de BASE :"
  u8g.drawHLine(12,10,82);
  u8g.setPrintPos(0,23); Aff_TEXTE_EEPROM(375, 21); // "BPC : Ouvre l'option."
  u8g.setPrintPos(18,36); Aff_TEXTE_EEPROM(396, 17); u8g.print('.'); // "Sortie par le BPC."
  u8g.setPrintPos(0,49); Aff_TEXTE_EEPROM(413, 20); // "Rot : Indexe option."
  u8g.setPrintPos(0,62); Aff_TEXTE_EEPROM(433, 20);} // "TIR & Dem : Ignorés."
  while(u8g.nextPage());}

void Page_kilometrique() {
  FINI = false;
  //---------------------- Traite le B.P. central ----------------------
  while(!FINI) {
    u8g.firstPage(); do {
    u8g.drawFrame(0,0,128,64); u8g.drawFrame(2,2,124,60); u8g.drawHLine(3,38,122);
    u8g.setPrintPos(11,17); Aff_TEXTE_EEPROM(523, 18); // "Nombre d'écritures");
    u8g.setPrintPos(32,33); Aff_TEXTE_EEPROM(541, 11); // "en EEPROM :");
    u8g.setPrintPos(44,54); u8g.print(NB_Ecritures_EEPROM);}
    while(u8g.nextPage());
    //================= Chronométrage pour le délai écoulé =============================
    Incremente_Energie_toutes_les_trois_secondes();
    //==================================================================================
    Teste_activation_BP_central();
    if (BP_central_a_traiter) {BP_central_a_traiter = false; FINI = true;}}}

void MENU_des_commandes() {
  FINI = false; Index = 1;
  if (Rotation) {// Ci-dessous pour annuler la fausse détection.
    INT_active = true; // Armer un nouveau comptage.
    Rotation = false;} // Prise en compte du traitement.
  while(!FINI) {
    //================= Chronométrage pour le délai écoulé =============================
    Incremente_Energie_toutes_les_trois_secondes();
    //==================================================================================
    switch (Index) {
      case 1 : {Ecran_1(); break;}
      case 2 : {Ecran_2(); break;}
      case 3 : Ecran_3();}
    //--------------- Traite la rotation du codeur incrémental -----------
    if (Rotation) {
    INT_active = true; // Armer un nouveau comptage.
    Rotation = false; // Prise en compte du traitement.
    if (Sens_positif) {if (Index > 1) Index--; else Index = 3;}
    else {Index++; if (Index == (4)) Index = 1;}}
    //---------------------- Traite le B.P. central ----------------------
    Teste_activation_BP_central();
    if (BP_central_a_traiter) {BP_central_a_traiter = false; FINI = true; Position_Index = 0;}}}

void Ecran_EXPLOITATION() {
  u8g.firstPage(); do {
    u8g.drawFrame(0,0,128,64); u8g.drawHLine(1,15,127); u8g.drawHLine(1,30,127);
    u8g.drawHLine(1,45,127); u8g.drawHLine(5,53,119); u8g.drawHLine(5,59,119);
    for (byte I = 0; I < 21; I++) u8g.drawVLine(4 + (6 * I),49,4);
    u8g.drawVLine(4,53,7); u8g.drawVLine(124,53,7);
    u8g.setPrintPos(24,12);
    if (Refroidissement) Aff_TEXTE_EEPROM(141, 15); // "REFROIDISSEMENT"
    else if (Cycle_en_cours) Aff_TEXTE_EEPROM(508, 15); // "Cycle en cours."
         else Aff_TEXTE_EEPROM(156, 15); // "ATTENTE de Dem."
    u8g.setPrintPos(19,27); Aff_TEXTE_EEPROM(171, 13); // "TIR validé : "
    OUI_NON(TIR_valide);
    u8g.setPrintPos(7,42); Aff_TEXTE_EEPROM(184, 19); // "> TMP :     / 118 <"
    u8g.setPrintPos(56,42); if (Temperature < 10) u8g.print('0');
    if (Temperature < 100) u8g.print('0'); u8g.print(Temperature);
    u8g.drawBox(5,54,Temperature,5);}
    while(u8g.nextPage());}

void Bruiteur() {if (!Silence) {tone(Buzzer, Frequence); Frequence = Frequence + 15;}}

void ACCELERE() {
  digitalWrite(Bobine1, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine1, LOW);
  digitalWrite(Bobine2, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine2, LOW);
  digitalWrite(Bobine3, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine3, LOW);
  digitalWrite(Bobine4, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine4, LOW);
  digitalWrite(Bobine5, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine5, LOW);
  digitalWrite(Bobine6, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine6, LOW);
  digitalWrite(Bobine7, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine7, LOW);
  digitalWrite(Bobine8, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine8, LOW);}

void Effectue_un_cycle() {
  Frequence = 20;
  Duree_un = 15; ACCELERE();
  Duree_un = 14; for (byte I=0; I<5; I++) ACCELERE();
  Duree_un = 12; for (byte I=0; I<5; I++) ACCELERE();
  Duree_un = 10; for (byte I=0; I<60; I++) ACCELERE();
  if (TIR_valide) {if (!Silence) tone(Buzzer,2); digitalWrite(TIR, HIGH); delay(500); digitalWrite(TIR, LOW);}
  noTone(Buzzer);} 

void Traiter_un_cycle() {
  Cycle_en_cours = true; Ecran_EXPLOITATION();
  Energie = Energie + 0.0352; Energie_totale = (eeprom_read_float((float *)1016)) + 0.0352;
  eeprom_write_float((float *)1016,Energie_totale); NB_Ecritures_EEPROM++;
  eeprom_write_dword((unsigned long *) 1012,NB_Ecritures_EEPROM);
  Eteindre_LEDs(); digitalWrite(VERTE, HIGH);
  if (TIR_valide) {Tir_depuis_RESET++; Nb_TIRs++;}; 
  Nb_Cycles++; Cycles_depuis_RESET++;
  eeprom_write_word((unsigned int *) 1020,Nb_TIRs);
  eeprom_write_word((unsigned int *) 1022,Nb_Cycles);
  Effectue_un_cycle();
  digitalWrite(VERTE, LOW); Cycle_en_cours = false;
  Temperature = Temperature + 6; if (Temperature > 119) Temperature = 119;}

void Ecran_Comptages() {
  u8g.firstPage(); do {
    u8g.drawFrame(0,0,128,64); u8g.drawFrame(2,2,124,60); u8g.drawHLine(3,25,122);
    u8g.setPrintPos(23,18); Aff_TEXTE_EEPROM(453, 15); // "Depuis le RESET"
    u8g.setPrintPos(38,40); if(Cycles_depuis_RESET < 10) u8g.print('0');
    u8g.print(Cycles_depuis_RESET); Aff_TEXTE_EEPROM(468, 7); // " Cycles"
    u8g.setPrintPos(38,53); if(Tir_depuis_RESET < 10) u8g.print('0');
    u8g.print(Tir_depuis_RESET); Aff_TEXTE_EEPROM(475, 5);} // " TIRs"
    while(u8g.nextPage());}
    
void Ecran_Energie() {
 u8g.firstPage(); do {
    u8g.drawFrame(0,0,128,64); u8g.drawFrame(2,2,124,60); u8g.drawHLine(3,25,122);
    u8g.setPrintPos(22,18); Aff_TEXTE_EEPROM(453, 15); // "Depuis le RESET"
    u8g.setPrintPos(8,40); Aff_TEXTE_EEPROM(491, 17); u8g.print(" :"); // "Energie consommée :"
    u8g.setPrintPos(36,53); u8g.print(Energie,4); Aff_TEXTE_EEPROM(99, 4);} // " Wh");}
    while(u8g.nextPage());}
    
void EXPLOITATION() {
  Mode_EXPLOITATION = true;
  //================= Chronométrage pour le délai écoulé =============================
  Incremente_Energie_toutes_les_trois_secondes();
  //==================================================================================
  switch (Index) {
    case 0 : {u8g.sleepOn(); Eteindre_LEDs(); digitalWrite(BLEUE, LOW); break;}
    case 1 : {u8g.sleepOff(); digitalWrite(BLEUE, HIGH); Ecran_EXPLOITATION(); break;}
    case 2 : {Ecran_Comptages(); break;}
    case 3 : {u8g.sleepOff(); Ecran_Energie(); Eteindre_LEDs(); digitalWrite(BLEUE, HIGH);}}
  //----------- Traite le BP central du codeur rotatif -----------
  Teste_activation_BP_central();
  if (BP_central_a_traiter) {
      BP_central_a_traiter = false; Silence = !Silence;
      u8g.firstPage(); do {
      u8g.drawRFrame(11,21,100,17,3); u8g.drawRFrame(7,18,108,23,5);
      u8g.setPrintPos(23,33); Aff_TEXTE_EEPROM(481, 10); OUI_NON(Silence);}
    while(u8g.nextPage()); BIP(); delay(2000);}
  //------------------ Traite le codeur rotatif ------------------
  if (Rotation) {INT_active = true; // Armer un nouveau comptage.
                 Rotation = false; // Prise en compte du traitement.
                 if (Sens_positif) {if (Index > 0) Index--; else Index = 3;}
                 else {Index++; if (Index == 4) Index = 0;}}
  //---------------------- Traite le BP TIR ----------------------
  if (analogRead(BP_Tir) < 512) {TIR_valide = !TIR_valide;
      while (analogRead(BP_Tir) < 512);} // Attendre le relâcher.
  //---------------------- Traite le BP DEM. ---------------------
  if (analogRead(BP_Marche) < 512) {
      while (analogRead(BP_Marche) < 512); // Attendre le relâcher.
      Traiter_un_cycle();
      if (Temperature == 119) {
        Refroidissement = true; Eteindre_LEDs(); digitalWrite(ROUGE, HIGH);
        // Ci-dessous le délai pour refroidir est d'environ 6 minutes.
        // T = 119 * 3 = 357 Secondes soit 5,95 minutes.
        while (Temperature > 1) {Ecran_EXPLOITATION(); delay(3000); Temperature--;}
      digitalWrite(ROUGE, LOW); Refroidissement = false;}}}
  
/* Problèmes liés au codeur rotatif :
   ----------------------------------
   Ce type de codeur engendre une rotation fantôme positive et négative à sa première utilisation.
   Pour parer cette difficulté, les instructions suivantes sont ajoutées dans les initialisations :
   Front_sur_A(); Front_sur_B(); Num_PGM_indexee = 4;
   Cette ligne de code effectue une "rotation positive artificielle". Du coup Num_PGM_indexee prend
   la valeur 0 en tête de listage et le fonctionnement du codeur rotatif est correct. 
   En fonction du codeur rotatif approvisionné, il peut s'avérer nécessaire d'inverser D2 et D3
   pour obtenir le sens de déplacement correct. */



