/* Programme complet d'exploitation d'une machine simplifiée sans afficheur OLED ni codeur rotatif.

Branchements à effectuer :
--------------------------
 A0 : LED Verte.
 A1 : LED Rouge.
 A2 : LED Bleue.

 D4  : Pilotage de la bobine B1.
 D5  : Pilotage de la bobine B2.
 D6  : Pilotage de la bobine B3.
 D7  : Pilotage de la bobine B4.
 D8  : Pilotage de la bobine B5.
 D9  : Pilotage de la bobine B6.
 D10  : Pilotage de la bobine B7.
 D12  : Pilotage de la bobine B8.
 D13  : Collisionneur.
                                                   PGM               RAM dynamique
Compilateur Version 1.7.9                    3650 Octets (11%)     43 Octets (2%) */

//------------- Définition des broches d'entrées analogiques ----------------
#define BP_Dem 20 // A6 pour lire le BP Dem.
#define BP_Tir 21 // A7 pour lire le BP Tir.

//------------- Définition des broches de sorties analogiques ---------------
#define VERTE 14 // A0 pour piloter la LED Verte.
#define ROUGE 15 // A1 pour piloter la LED Rouge.
#define BLEUE 16 // A2 pour piloter la LED Bleue.
//-------------- Définition des broches de sorties Binaires -----------------
#define Bobine1 4    // D4 : Pilote B1.
#define Bobine2 5    // D5 : Pilote B2.
#define Bobine3 6    // D6 : Pilote B3.
#define Bobine4 7    // D7 : Pilote B4.
#define Bobine5 8    // D8 : Pilote B5.
#define Bobine6 9    // D9 : Pilote B6.
#define Bobine7 10   // D10 : Pilote B7.
#define Buzzer  11   // D11 : Bruiteur passif.
#define Bobine8 12   // D12 : Pilote B8.
#define TIR 13       // D13 : Pilote le collisionneur.

//----------------------- Déclaration des variables -------------------------

boolean Refroidissement, TIR_valide;

byte Temperature;

int Duree_un; // Durées en centièmes de seconde.
int Frequence;

unsigned long Temps_Ecoule_depuis_RESET = 0;

//---------------------------------------------------------------------------

void setup() {
  //---------------------- Définition des vecteurs d'interruption ----------------------
  pinMode(BLEUE, OUTPUT); pinMode(VERTE, OUTPUT); pinMode(ROUGE, OUTPUT);
  digitalWrite(BLEUE, LOW); digitalWrite(VERTE, LOW); digitalWrite(ROUGE, LOW);
  pinMode(Bobine1, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine2, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine3, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine4, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine5, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine6, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine7, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine8, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(TIR, OUTPUT); digitalWrite(TIR, LOW);
  TIR_valide = false; Temperature = 1; Refroidissement = false;}

void loop() {EXPLOITATION();}

void BIP() {tone(Buzzer, 3000, 150);}

void Bruiteur() {tone(Buzzer, Frequence); Frequence = Frequence + 15;}

void ACCELERE() {
  digitalWrite(Bobine1, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine1, LOW);
  digitalWrite(Bobine2, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine2, LOW);
  digitalWrite(Bobine3, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine3, LOW);
  digitalWrite(Bobine4, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine4, LOW);
  digitalWrite(Bobine5, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine5, LOW);
  digitalWrite(Bobine6, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine6, LOW);
  digitalWrite(Bobine7, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine7, LOW);
  digitalWrite(Bobine8, HIGH); Bruiteur(); delay(Duree_un); digitalWrite(Bobine8, LOW);}

void Effectue_un_cycle() {
  Frequence = 20;
  Duree_un = 15; ACCELERE();
  Duree_un = 14; for (byte I=0; I<5; I++) ACCELERE();
  Duree_un = 12; for (byte I=0; I<5; I++) ACCELERE();
  Duree_un = 10; for (byte I=0; I<60; I++) ACCELERE();
  if (TIR_valide) {tone(Buzzer,2); digitalWrite(TIR, HIGH); delay(500); digitalWrite(TIR, LOW);}
  noTone(Buzzer);} 

void Traiter_un_cycle() {
  Effectue_un_cycle();
  Temperature = Temperature + 6; if (Temperature > 119) Temperature = 119;}
  
void EXPLOITATION() {
  digitalWrite(BLEUE, HIGH);
  //================= Chronométrage pour le délai écoulé =============================
  if (millis() - Temps_Ecoule_depuis_RESET > 3000)  {
     Temps_Ecoule_depuis_RESET = millis(); // réarme un prochain comptage.
     //----------------- Actions à traiter pour chaque délai écoulé. -----------------
     if (Temperature > 1) Temperature--;}
  //==================================================================================
  //---------------------- Traite le BP TIR ----------------------
  if (analogRead(BP_Tir) < 512) {TIR_valide = !TIR_valide; BIP();
      while (analogRead(BP_Tir) < 512); // Attendre le relâcher.
      if (TIR_valide) digitalWrite(VERTE, HIGH); else digitalWrite(VERTE, LOW);}
  //---------------------- Traite le BP DEM. ---------------------
  if (analogRead(BP_Dem) < 512) {
      digitalWrite(BLEUE, LOW);
      while (analogRead(BP_Dem) < 512); // Attendre le relâcher.
      Traiter_un_cycle();
      if (Temperature == 119) {
        Refroidissement = true; digitalWrite(VERTE, LOW); digitalWrite(ROUGE, HIGH);
        // Ci-dessous le délai pour refroidir est d'environ 6 minutes.
        // T = 119 * 3 = 357 Secondes soit 5,95 minutes.
        while (Temperature > 1) {delay(3000); Temperature--;}
        digitalWrite(ROUGE, LOW); Refroidissement = false;}}}


